"""
Module palette.py

This module contains the color style references for the graphics

Original author: Andrea Gimelli, Giacomo Rosato, Stefano Demarchi

"""

DARK_TEAL = "#105555"
TEAL = "#157172"
LIGHT_TEAL = "#1ABC9C"

GREEN_1 = "#0A6160"
GREEN_2 = "#0C8371"
LIGHT_GREEN = "#27AE60"
DARK_GREEN = "#1E8449"

RED = "#b80000"
RED_1 = "#FF0055"
RED_2 = "#E60000"
LIGHT_RED = "#E74C3C"
DARK_RED = "#C0392B"

DARK_BLUE = "#304153"
BLUE = "#34475b"
BLUE_0 = "#005588"
BLUE_1 = "#0077AA"

DARK_GREY = "#2c2c2c"
LESS_DARK_GREY = "#303030"
GREY = "#4B4E52"
LIGHT_GREY = "#56595d"
GREY_0 = "#2A2A2A"
GREY_1 = "#323232"
GREY_2 = "#4D4D4D"
GREY_3 = "#656565"
GREY_4 = "#6F6F6F"

RED = "#b80000"
RED_1 = "#FF0055"
RED_2 = "#E60000"

DARK_ORANGE = "#873511"
ORANGE = "#943A12"
ORANGE_0 = "#FF9900"
ORANGE_1 = "#FFAD33"

BACKGROUND_GREY = "#393939"
BACKGROUND_LIGHT_LINE_GREY = "#2f2f2f"
BACKGROUND_DARK_LINE_GREY = "#292929"

BLACK = "black"
WHITE = "white"
WHITE_GRAY = "#F2F2F2"

VIOLET   = "#7C3AED"
VIOLET_1 = "#8B5CF6"
