"""
Styling module for the node editor interface.

This module provides styling constants, dimensions, and color palette.

"""

from . import dimension
from . import palette

__all__ = [
    'dimension', 'palette'
]
